<?php

// Formule 1 race - Gripp

// For more information see the API documentation:
// https://yourdomain.gripp.com/public/api2.php

// Classes and configuration files that are needed. 
require_once('config.php');
require_once('lib/api.class.php');
require_once('lib/SimpleTemplateParser.class.php');

$api = new com_gripp_API($apikey, $url);

//Depending on the settings, use a different day and time.
if (isset($_GET['dag'])){
    $startdate = new \DateTime();
    $stopdate = new \DateTime();
}
else{
    $startdate = new \DateTime();
    if ($startdate->format('N') != 1){
        $startdate->modify('last monday');
    }
    $stopdate = clone $startdate;
    $stopdate->modify('+6 day');
}



// Get all hours that are made on the specified date.
$filters = array(
    array(
        "field" => "hour.date",
        "operator" => "between",
        "value" => $startdate->format('Y-m-d'),
        "value2" => $stopdate->format('Y-m-d')
    )
);

$options = array(
    "paging" => array(
        "firstresult" => 0,
        "maxresults" => 250
    ),
    "orderings" => array(
        array(
            "field" => "hour.id",
            "direction" => "asc"
        )
    )
);

$rows = array();

// Get all the hours(rows), do this in multiple requests if needed, because of the API paging (see the API documentation for more information.)
while(true) {
	$responses = $api->hour_get($filters, $options);
	$response = $responses[0]['result'];

	$rows = array_merge($rows, $response['rows']);
	
	if($response['more_items_in_collection'] == 1) {
		$options['paging']['firstresult'] = $response['next_start'];
	} else {
		break;
	}
}

//Counting the hours per employeeID.
$totals_per_employee = array();

foreach($rows as $hour){
    $employee_id = $hour['employee']['id'];
    if (!array_key_exists($employee_id, $totals_per_employee)){
        $totals_per_employee[$employee_id] = array(
            'id' => $employee_id,
            'hours' => array(),
            'sum' => 0,
            'latestid' => null
        );
    }

    if ($hour['id'] > $totals_per_employee[$employee_id]['latestid']){
        $totals_per_employee[$employee_id]['latestid'] = $hour['id'];
    }
    $totals_per_employee[$employee_id]['sum'] += $hour['amount'];
}

//fetching the employees
$filters = array(
    array(
        "field" => "employee.active",
        "operator" => "equals",
        "value" => true
    )
);
$options = array(
    "paging" => array(
        "firstresult" => 0,
        "maxresults" => 250
    ),
    "orderings" => array(
        array(
            "field" => "employee.searchname",
            "direction" => "asc"
        )
    )
);

// Get all the employees(rows), do this in multiple requests if needed, because of the API paging (see the API documentation for more information.)
$employees = array();
while(true) {
	$responses = $api->employee_get($filters, $options);
	$response = $responses[0]['result'];

	$employees = array_merge($employees, $response['rows']);
	
	if($response['more_items_in_collection'] == 1) {
		$options['paging']['firstresult'] = $response['next_start'];
	} else {
		break;
	}
}

$photoids = array();
$employee_ids = array();
foreach($employees as $i => $employee){
    $employee_ids[] = $employee['id'];
    if (array_key_exists('userphoto', $employee) && is_array($employee['userphoto'])){
        $photoids[] = $employee['userphoto']['id'];
    }
}

//Fetching the workinghours
$responses = $api->employee_getWorkingHours($employee_ids, $startdate->format('Y-m-d'), $stopdate->format('Y-m-d'));
$response = $responses[0]['result'];
$workingHoursPerEmployee = $response;
foreach($employees as $i => $employee){
    $employees[$i]['totalplannedhoursaweek'] = $workingHoursPerEmployee[$employee['id']]['sum'];
}


//fetching the photos
$filters = array(
    array(
        "field" => "file.id",
        "operator" => "in",
        "value" => $photoids
    )
);
$options = array(
    "paging" => array(
        "firstresult" => 0,
        "maxresults" => 250
    ),
    "orderings" => array(
        array(
            "field" => "file.id",
            "direction" => "asc"
        )
    )
);

$photos = array();

while(true) {
    $responses = $api->file_get($filters, $options);
    $response = $responses[0]['result'];

    $photos = array_merge($photos, $response['rows']);
    
    if($response['more_items_in_collection'] == 1) {
        $options['paging']['firstresult'] = $response['next_start'];
    } else {
        break;
    }
}

$photos_by_id = array();
foreach($photos as $photo){
    $photos_by_id[$photo['id']] = $photo;
}

//We now have everything we need. Reformat to fit clientside needs.
$combineddata = array();
foreach($employees as $employee){
    $actualhours = array_key_exists($employee['id'], $totals_per_employee) ? $totals_per_employee[$employee['id']]['sum'] : 0;
    $latestid = array_key_exists($employee['id'], $totals_per_employee) ? $totals_per_employee[$employee['id']]['latestid'] : null;
    $percentage = ($employee['totalplannedhoursaweek'] == 0) ? 0 : (floatval($actualhours) / floatval($employee['totalplannedhoursaweek'])) * 100;
    $percentage = $percentage > 100 ? 100 : $percentage;
    $photo = ''; //todo default empty photo
    if (is_array($employee['userphoto'])){
        $photo = $photos_by_id[$employee['userphoto']['id']]['previewdatasmall'];
    }

    $combineddata[] = array(
        'employee_searchname' => $employee['searchname'],
        'totalplannedhoursaweek' => $employee['totalplannedhoursaweek'],
        'actualhours' => $actualhours,
        'percentage' => $percentage,
        'latestid' => $latestid,
        'photo' => $photo,
        'winner' => false
    );
}

//determine the winner
$winner = null;
foreach($combineddata as $i => $data){
    if ($data['actualhours'] > 0 && !is_null($data['totalplannedhoursaweek']) && $data['actualhours'] >= $data['totalplannedhoursaweek']){ //reached the minimum hours
        if (is_null($winner) || $data['latestid'] < $combineddata[$winner]['latestid']){
            $winner = $i;
        }
    }
}
if (!is_null($winner)){
    $combineddata[$winner]['winner'] = true;
}


//Fill the template and echo the result.
$templateparser = new SimpleTemplateParser();
$templateparser->assign('combineddata', $combineddata);
echo $templateparser->fetch('assets/index.tpl');